@@ -2,37 +2,36 @@
 from distutils and setuptools.
 """
 
-from __future__ import print_function
 
 import argparse
+import contextlib
 import copy
+import glob
+import io
 import json
 import os
 import os.path
 import platform
+import shutil
 import stat
 import sys
 import warnings
-from contextlib import contextmanager
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Union,
+)
 
-# pylint: disable-next=wrong-import-order
-from distutils.errors import DistutilsArgError, DistutilsError, DistutilsGetoptError
-from glob import glob
-from shutil import copyfile, copymode
-
-# Must be imported before distutils
 import setuptools
-
-if sys.version_info >= (3, 0):
-    from io import StringIO
-else:
-    from StringIO import StringIO
-
-if sys.version_info >= (3, 3):
-    from shutil import which
-else:
-    from .compat import which
-
+from distutils.errors import DistutilsArgError, DistutilsError, DistutilsGetoptError
 from packaging.requirements import Requirement
 from packaging.version import parse as parse_version
 from setuptools.dist import Distribution as upstream_Distribution
@@ -60,7 +59,11 @@ from .constants import (
     set_skbuild_plat_name,
     skbuild_plat_name,
 )
-from .exceptions import SKBuildError, SKBuildGeneratorNotFoundError
+from .exceptions import (
+    SKBuildError,
+    SKBuildGeneratorNotFoundError,
+    SKBuildInvalidFileInstallationError,
+)
 from .utils import (
     PythonModuleFinder,
     mkdir_p,
@@ -70,7 +73,7 @@ from .utils import (
 )
 
 
-def create_skbuild_argparser():
+def create_skbuild_argparser() -> argparse.ArgumentParser:
     """Create and return a scikit-build argument parser."""
     parser = argparse.ArgumentParser(add_help=False)
     parser.add_argument(
@@ -93,7 +96,7 @@ def create_skbuild_argparser():
     return parser
 
 
-def _is_cmake_configure_argument(arg):
+def _is_cmake_configure_argument(arg: str) -> bool:
     """Return True if ``arg`` is a relevant argument to pass to cmake when configuring a project."""
 
     for cmake_arg in (
@@ -105,7 +108,9 @@ def _is_cmake_configure_argument(arg):
     return False
 
 
-def parse_skbuild_args(args, cmake_args, build_tool_args):
+def parse_skbuild_args(
+    args: Sequence[str], cmake_args: List[str], build_tool_args: List[str]
+) -> Tuple[List[str], Optional[str], bool]:
     """
     Parse arguments in the scikit-build argument set. Convert specified
     arguments to proper format and append to cmake_args and build_tool_args.
@@ -138,13 +143,13 @@ def parse_skbuild_args(args, cmake_args, build_tool_args):
     return remaining_args, namespace.cmake_executable, namespace.skip_generator_test
 
 
-def parse_args():
+def parse_args() -> Tuple[List[str], Optional[str], bool, List[str], List[str]]:
     """This function parses the command-line arguments ``sys.argv`` and returns
     the tuple ``(setuptools_args, cmake_executable, skip_generator_test, cmake_args, build_tool_args)``
     where each ``*_args`` element corresponds to a set of arguments separated by ``--``."""
-    dutils = []
-    cmake = []
-    make = []
+    dutils: List[str] = []
+    cmake: List[str] = []
+    make: List[str] = []
     argsets = [dutils, cmake, make]
     i = 0
     separator = "--"
@@ -153,10 +158,7 @@ def parse_args():
         if arg == separator:
             i += 1
             if i >= len(argsets):
-                sys.exit(
-                    'ERROR: Too many "{}" separators provided '
-                    "(expected at most {}).".format(separator, len(argsets) - 1)
-                )
+                sys.exit(f"ERROR: Too many '{separator}' separators provided (expected at most {len(argsets) - 1}).")
         else:
             argsets[i].append(arg)
 
@@ -165,20 +167,24 @@ def parse_args():
     return dutils, cmake_executable, skip_generator_test, cmake, make
 
 
-@contextmanager
-def _capture_output():
-    oldout, olderr = sys.stdout, sys.stderr
-    try:
-        out = [StringIO(), StringIO()]
-        sys.stdout, sys.stderr = out
-        yield out
-    finally:
-        sys.stdout, sys.stderr = oldout, olderr
-        out[0] = out[0].getvalue()
-        out[1] = out[1].getvalue()
+@contextlib.contextmanager
+def _capture_output() -> Generator[List[Union[io.StringIO, str]], None, None]:
+    out: List[Union[io.StringIO, str]]
+
+    with contextlib.redirect_stdout(io.StringIO()) as stdout:
+        with contextlib.redirect_stderr(io.StringIO()) as stderr:
+            out = [stdout, stderr]
+            yield out
+
+    assert isinstance(out[0], io.StringIO)
+    assert isinstance(out[1], io.StringIO)
+    out[0] = out[0].getvalue()
+    out[1] = out[1].getvalue()
 
 
-def _parse_setuptools_arguments(setup_attrs):
+def _parse_setuptools_arguments(
+    setup_attrs: Mapping[str, Any]
+) -> Tuple[bool, bool, List[str], bool, bool, bool, str, bool]:
     """This function instantiates a Distribution object and
     parses the command line arguments.
 
@@ -215,9 +221,9 @@ def _parse_setuptools_arguments(setup_attrs):
 
     # Update class attribute to also ensure the argument is processed
     # when ``setuptools.setup`` is called.
-    upstream_Distribution.global_options.extend(
+    upstream_Distribution.global_options.extend(  # type: ignore[attr-defined]
         [
-            ("hide-listing", None, "do not display list of files being " "included in the distribution"),
+            ("hide-listing", None, "do not display list of files being included in the distribution"),
             ("force-cmake", None, "always run CMake"),
             ("skip-cmake", None, "do not run CMake"),
         ]
@@ -232,64 +238,64 @@ def _parse_setuptools_arguments(setup_attrs):
     # SystemExit to suppress tracebacks.
 
     with _capture_output():
-        result = dist.parse_command_line()
+        result = dist.parse_command_line()  # type: ignore[attr-defined]
         display_only = not result
         if not hasattr(dist, "hide_listing"):
-            dist.hide_listing = False
+            dist.hide_listing = False  # type: ignore[attr-defined]
         if not hasattr(dist, "force_cmake"):
-            dist.force_cmake = False
+            dist.force_cmake = False  # type: ignore[attr-defined]
         if not hasattr(dist, "skip_cmake"):
-            dist.skip_cmake = False
+            dist.skip_cmake = False  # type: ignore[attr-defined]
 
     plat_names = set()
-    for cmd in [dist.get_command_obj(command) for command in dist.commands]:
-        if getattr(cmd, "plat_name", None) is not None:
-            plat_names.add(cmd.plat_name)
+    for cmd in [dist.get_command_obj(command) for command in dist.commands]:  # type: ignore[attr-defined]
+        plat_name = getattr(cmd, "plat_name", None)
+        if plat_name is not None:
+            plat_names.add(plat_name)
     if not plat_names:
         plat_names.add(None)
     elif len(plat_names) > 1:
-        raise SKBuildError("--plat-name is ambiguous: %s" % ", ".join(plat_names))
+        names = ", ".join(plat_names)
+        msg = f"--plat-name is ambiguous: {names}"
+        raise SKBuildError(msg)
     plat_name = list(plat_names)[0]
 
-    build_ext_inplace = dist.get_command_obj("build_ext").inplace
+    build_ext_cmd = dist.get_command_obj("build_ext")
+    build_ext_inplace: bool = getattr(build_ext_cmd, "inplace", False)
 
     return (
         display_only,
-        dist.help_commands,
-        dist.commands,
-        dist.hide_listing,
-        dist.force_cmake,
-        dist.skip_cmake,
+        dist.help_commands,  # type: ignore[attr-defined]
+        dist.commands,  # type: ignore[attr-defined]
+        dist.hide_listing,  # type: ignore[attr-defined]
+        dist.force_cmake,  # type: ignore[attr-defined]
+        dist.skip_cmake,  # type: ignore[attr-defined]
         plat_name,
         build_ext_inplace,
     )
 
 
-def _check_skbuild_parameters(skbuild_kw):
-    cmake_install_dir = skbuild_kw["cmake_install_dir"]
+def _check_skbuild_parameters(cmake_install_dir: str, cmake_source_dir: str) -> None:
     if os.path.isabs(cmake_install_dir):
-        raise SKBuildError(
-            (
-                "\n  setup parameter 'cmake_install_dir' is set to "
-                "an absolute path. A relative path is expected.\n"
-                "    Project Root  : {}\n"
-                "    CMake Install Directory: {}\n"
-            ).format(os.getcwd(), cmake_install_dir)
+        msg = (
+            "\n  setup parameter 'cmake_install_dir' is set to "
+            "an absolute path. A relative path is expected.\n"
+            f"    Project Root  : {os.getcwd()}\n"
+            f"    CMake Install Directory: {cmake_install_dir}\n"
         )
+        raise SKBuildError(msg)
 
-    cmake_source_dir = skbuild_kw["cmake_source_dir"]
     if not os.path.exists(os.path.abspath(cmake_source_dir)):
-        raise SKBuildError(
-            (
-                "\n  setup parameter 'cmake_source_dir' set to "
-                "a nonexistent directory.\n"
-                "    Project Root  : {}\n"
-                "    CMake Source Directory: {}\n"
-            ).format(os.getcwd(), cmake_source_dir)
+        msg = (
+            "\n  setup parameter 'cmake_source_dir' set to "
+            "a nonexistent directory.\n"
+            f"    Project Root  : {os.getcwd()}\n"
+            f"    CMake Source Directory: {cmake_source_dir}\n"
         )
+        raise SKBuildError(msg)
 
 
-def strip_package(package_parts, module_file):
+def strip_package(package_parts: Sequence[str], module_file: str) -> str:
     """Given ``package_parts`` (e.g. ``['foo', 'bar']``) and a
     ``module_file`` (e.g. ``foo/bar/jaz/rock/roll.py``), starting
     from the left, this function will strip the parts of the path
@@ -310,7 +316,7 @@ def strip_package(package_parts, module_file):
     return module_file[len(package) + 1 :] if package != "" and module_dir.startswith(package) else module_file
 
 
-def _package_data_contain_module(module, package_data):
+def _package_data_contain_module(module: Tuple[str, str, str], package_data: Mapping[str, List[str]]) -> bool:
     """Return True if the ``module`` is contained
     in the ``package_data``.
 
@@ -333,10 +339,11 @@ def _package_data_contain_module(module, package_data):
     return False
 
 
-def _should_run_cmake(commands, cmake_with_sdist):
+def _should_run_cmake(commands: Sequence[str], cmake_with_sdist: bool) -> bool:
     """Return True if at least one command requiring ``cmake`` to run
     is found in ``commands``."""
-    for expected_command in [
+    given_commands = set(commands)
+    expected_commands = {
         "build",
         "build_ext",
         "develop",
@@ -349,37 +356,45 @@ def _should_run_cmake(commands, cmake_with_sdist):
         "bdist_wininst",
         "bdist_wheel",
         "test",
-    ]:
-        if expected_command in commands:
-            return True
-    if "sdist" in commands and cmake_with_sdist:
+    }
+
+    if expected_commands & given_commands:
+        return True
+    if "sdist" in given_commands and cmake_with_sdist:
         return True
     return False
 
 
-def _save_cmake_spec(args):
+def _save_cmake_spec(args: Mapping[str, Any]) -> None:
     """Save the CMake spec to disk"""
     # We use JSON here because readability is more important than performance
-    try:
+    with contextlib.suppress(OSError):
         os.makedirs(os.path.dirname(CMAKE_SPEC_FILE()))
-    except OSError:
-        pass
 
-    with open(CMAKE_SPEC_FILE(), "w+") as fp:
+    with open(CMAKE_SPEC_FILE(), "w+", encoding="utf-8") as fp:
         json.dump(args, fp)
 
 
-def _load_cmake_spec():
+def _load_cmake_spec() -> Any:
     """Load and return the CMake spec from disk"""
-    try:
-        with open(CMAKE_SPEC_FILE()) as fp:
-            return json.load(fp)
-    except (OSError, IOError, ValueError):
-        return None
+    with contextlib.suppress(OSError, ValueError), open(CMAKE_SPEC_FILE(), encoding="utf-8") as fp:
+        return json.load(fp)
+    return None
 
 
 # pylint:disable=too-many-locals, too-many-branches
-def setup(*args, **kw):  # noqa: C901
+def setup(  # noqa: C901
+    *,
+    cmake_args: Sequence[str] = (),
+    cmake_install_dir: str = "",
+    cmake_source_dir: str = "",
+    cmake_with_sdist: bool = False,
+    cmake_languages: Sequence[str] = ("C", "CXX"),
+    cmake_minimum_required_version: Optional[str] = None,
+    cmake_process_manifest_hook: Optional[Callable[[List[str]], List[str]]] = None,
+    cmake_install_target: str = "install",
+    **kw: Any,
+) -> None:
     """This function wraps setup() so that we can run cmake, make,
     CMake build, then proceed as usual with setuptools, appending the
     CMake-generated output as necessary.
@@ -396,13 +411,11 @@ def setup(*args, **kw):  # noqa: C901
     if "package_dir" in kw:
         for package, prefix in kw["package_dir"].items():
             if prefix.endswith("/"):
-                msg = "package_dir={{{!r}: {!r}}} ends with a trailing slash, which is not supported by setuptools.".format(
-                    package, prefix
-                )
+                msg = f"package_dir={{{package!r}: {prefix!r}}} ends with a trailing slash, which is not supported by setuptools."
                 warnings.warn(msg, FutureWarning, stacklevel=2)
                 kw["package_dir"][package] = prefix[:-1]
 
-    sys.argv, cmake_executable, skip_generator_test, cmake_args, make_args = parse_args()
+    sys.argv, cmake_executable, skip_generator_test, cmake_args_from_args, make_args = parse_args()
 
     # work around https://bugs.python.org/issue1011113
     # (patches provided, but no updates since 2014)
@@ -428,32 +441,20 @@ def setup(*args, **kw):  # noqa: C901
     # Removing the keyword from kw need to be done here otherwise, the
     # following call to _parse_setuptools_arguments would complain about
     # unknown setup options.
-    parameters = {
-        "cmake_args": [],
-        "cmake_install_dir": "",
-        "cmake_source_dir": "",
-        "cmake_with_sdist": False,
-        "cmake_languages": ("C", "CXX"),
-        "cmake_minimum_required_version": None,
-        "cmake_process_manifest_hook": None,
-        "cmake_install_target": "install",
-    }
-    skbuild_kw = {param: kw.pop(param, value) for param, value in parameters.items()}
 
     # ... and validate them
     try:
-        _check_skbuild_parameters(skbuild_kw)
+        _check_skbuild_parameters(cmake_install_dir, cmake_source_dir)
     except SKBuildError as ex:
         import traceback  # pylint: disable=import-outside-toplevel
 
-        print("Traceback (most recent call last):")
+        print("Traceback (most recent call last):", file=sys.stderr)
         traceback.print_tb(sys.exc_info()[2])
-        print("")
+        print(file=sys.stderr, flush=True)
         sys.exit(ex)
 
     # Convert source dir to a path relative to the root
     # of the project
-    cmake_source_dir = skbuild_kw["cmake_source_dir"]
     if cmake_source_dir == ".":
         cmake_source_dir = ""
     if os.path.isabs(cmake_source_dir):
@@ -467,7 +468,7 @@ def setup(*args, **kw):  # noqa: C901
     # * no CMakeLists.txt if found
     display_only = has_invalid_arguments = help_commands = False
     force_cmake = skip_cmake = False
-    commands = []
+    commands: List[str] = []
     try:
         (
             display_only,
@@ -481,16 +482,15 @@ def setup(*args, **kw):  # noqa: C901
         ) = _parse_setuptools_arguments(kw)
     except (DistutilsArgError, DistutilsGetoptError):
         has_invalid_arguments = True
+        plat_name = None
+        hide_listing = False
 
     has_cmakelists = os.path.exists(os.path.join(cmake_source_dir, "CMakeLists.txt"))
     if not has_cmakelists:
-        print("skipping skbuild (no CMakeLists.txt found)")
+        print("skipping skbuild (no CMakeLists.txt found)", flush=True)
 
     skip_skbuild = (
-        display_only
-        or has_invalid_arguments
-        or not _should_run_cmake(commands, skbuild_kw["cmake_with_sdist"])
-        or not has_cmakelists
+        display_only or has_invalid_arguments or not _should_run_cmake(commands, cmake_with_sdist) or not has_cmakelists
     )
     if skip_skbuild and not force_cmake:
         if help_commands:
@@ -500,11 +500,11 @@ def setup(*args, **kw):  # noqa: C901
             arg_descriptions = [line for line in skbuild_parser.format_help().split("\n") if line.startswith("  ")]
             print("scikit-build options:")
             print("\n".join(arg_descriptions))
-            print("")
+            print()
             print('Arguments following a "--" are passed directly to CMake ' "(e.g. -DMY_VAR:BOOL=TRUE).")
             print('Arguments following a second "--" are passed directly to ' " the build tool.")
-            print("")
-        return setuptools.setup(*args, **kw)
+            print(flush=True)
+        return setuptools.setup(**kw)
 
     developer_mode = "develop" in commands or "test" in commands or build_ext_inplace
 
@@ -523,14 +523,14 @@ def setup(*args, **kw):  # noqa: C901
     # Since CMake arguments provided through the command line have more
     # weight and when CMake is given multiple times a argument, only the last
     # one is considered, let's prepend the one provided in the setup call.
-    cmake_args = skbuild_kw["cmake_args"] + cmake_args
+    cmake_args = list(cmake_args) + cmake_args_from_args
 
     # Handle cmake_install_target
     # get the target (next item after '--install-target') or return '' if no --install-target
     cmake_install_target_from_command = next(
         (make_args[index + 1] for index, item in enumerate(make_args) if item == "--install-target"), ""
     )
-    cmake_install_target_from_setup = skbuild_kw["cmake_install_target"]
+    cmake_install_target_from_setup = cmake_install_target
     # Setting target from command takes precedence
     # cmake_install_target_from_setup has the default 'install',
     # so cmake_install_target would never be empty.
@@ -565,7 +565,7 @@ def setup(*args, **kw):  # noqa: C901
                 if set(machine.split(";")) == {"x86_64", "arm64"}:
                     machine = "universal2"
 
-        set_skbuild_plat_name("macosx-{}-{}".format(version, machine))
+        set_skbuild_plat_name(f"macosx-{version}-{machine}")
 
         # Set platform env. variable so that commands (e.g. bdist_wheel)
         # uses this information. The _PYTHON_HOST_PLATFORM env. variable is
@@ -576,17 +576,17 @@ def setup(*args, **kw):  # noqa: C901
         # specified
         (_, version, machine) = skbuild_plat_name().split("-")
         if not cmaker.has_cmake_cache_arg(cmake_args, "CMAKE_OSX_DEPLOYMENT_TARGET"):
-            cmake_args.append("-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=%s" % version)
+            cmake_args.append(f"-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING={version}")
         if not cmaker.has_cmake_cache_arg(cmake_args, "CMAKE_OSX_ARCHITECTURES"):
             machine_archs = "x86_64;arm64" if machine == "universal2" else machine
-            cmake_args.append("-DCMAKE_OSX_ARCHITECTURES:STRING=%s" % machine_archs)
+            cmake_args.append(f"-DCMAKE_OSX_ARCHITECTURES:STRING={machine_archs}")
 
     # Install cmake if listed in `setup_requires`
     for package in kw.get("setup_requires", []):
         if Requirement(package).name == "cmake":
             setup_requires = [package]
             dist = upstream_Distribution({"setup_requires": setup_requires})
-            dist.fetch_build_eggs(setup_requires)
+            dist.fetch_build_eggs(setup_requires)  # type: ignore[no-untyped-call]
 
             # Considering packages associated with "setup_requires" keyword are
             # installed in .eggs subdirectory without honoring setuptools "console_scripts"
@@ -594,7 +594,7 @@ def setup(*args, **kw):  # noqa: C901
             # taking care of it below.
             import cmake  # pylint: disable=import-outside-toplevel
 
-            for executable in ["cmake", "cpack", "ctest"]:
+            for executable in ("cmake", "cpack", "ctest"):
                 executable = os.path.join(cmake.CMAKE_BIN_DIR, executable)
                 if platform.system().lower() == "windows":
                     executable += ".exe"
@@ -605,25 +605,20 @@ def setup(*args, **kw):  # noqa: C901
             break
 
     # Languages are used to determine a working generator
-    cmake_languages = skbuild_kw["cmake_languages"]
 
     try:
         if cmake_executable is None:
             cmake_executable = CMAKE_DEFAULT_EXECUTABLE
         cmkr = cmaker.CMaker(cmake_executable)
         if not skip_cmake:
-            cmake_minimum_required_version = skbuild_kw["cmake_minimum_required_version"]
             if cmake_minimum_required_version is not None:
                 if parse_version(cmkr.cmake_version) < parse_version(cmake_minimum_required_version):
-                    raise SKBuildError(
-                        "CMake version {} or higher is required. CMake version {} is being used".format(
-                            cmake_minimum_required_version, cmkr.cmake_version
-                        )
-                    )
+                    msg = f"CMake version {cmake_minimum_required_version} or higher is required. CMake version {cmkr.cmake_version} is being used"
+                    raise SKBuildError(msg)
             # Used to confirm that the cmake executable is the same, and that the environment
             # didn't change
             cmake_spec = {
-                "args": [which(CMAKE_DEFAULT_EXECUTABLE)] + cmake_args,
+                "args": [shutil.which(CMAKE_DEFAULT_EXECUTABLE)] + cmake_args,
                 "version": cmkr.cmake_version,
                 "environment": {
                     "PYTHONNOUSERSITE": os.environ.get("PYTHONNOUSERSITE"),
@@ -638,7 +633,7 @@ def setup(*args, **kw):  # noqa: C901
                     cmake_args,
                     skip_generator_test=skip_generator_test,
                     cmake_source_dir=cmake_source_dir,
-                    cmake_install_dir=skbuild_kw["cmake_install_dir"],
+                    cmake_install_dir=cmake_install_dir,
                     languages=cmake_languages,
                 )
                 _save_cmake_spec(cmake_spec)
@@ -648,9 +643,9 @@ def setup(*args, **kw):  # noqa: C901
     except SKBuildError as ex:
         import traceback  # pylint: disable=import-outside-toplevel
 
-        print("Traceback (most recent call last):")
+        print("Traceback (most recent call last):", file=sys.stderr)
         traceback.print_tb(sys.exc_info()[2])
-        print("")
+        print(file=sys.stderr, flush=True)
         sys.exit(ex)
 
     # If needed, set reasonable defaults for package_dir
@@ -666,7 +661,7 @@ def setup(*args, **kw):  # noqa: C901
 
     # This hook enables custom processing of the cmake manifest
     cmake_manifest = cmkr.install()
-    process_manifest = skbuild_kw.get("cmake_process_manifest_hook")
+    process_manifest = cmake_process_manifest_hook
     if process_manifest is not None:
         if callable(process_manifest):
             cmake_manifest = process_manifest(cmake_manifest)
@@ -683,7 +678,7 @@ def setup(*args, **kw):  # noqa: C901
         new_scripts,
         data_files,
         cmake_source_dir,
-        skbuild_kw["cmake_install_dir"],
+        cmake_install_dir,
     )
 
     original_manifestin_data_files = []
@@ -716,7 +711,7 @@ def setup(*args, **kw):  # noqa: C901
         _consolidate_package_data_files(original_package_data, package_prefixes, hide_listing)
 
         for data_file in original_manifestin_data_files:
-            dest_data_file = os.path.join(CMAKE_INSTALL_DIR(), skbuild_kw["cmake_install_dir"], data_file)
+            dest_data_file = os.path.join(CMAKE_INSTALL_DIR(), data_file)
             _copy_file(data_file, dest_data_file, hide_listing)
 
     kw["package_data"] = package_data
@@ -740,17 +735,19 @@ def setup(*args, **kw):  # noqa: C901
 
     # Adapted from espdev/ITKPythonInstaller/setup.py.in
     class BinaryDistribution(upstream_Distribution):  # pylint: disable=missing-class-docstring
-        def has_ext_modules(self):  # pylint: disable=no-self-use,missing-function-docstring
+        def has_ext_modules(self) -> bool:  # pylint: disable=no-self-use,missing-function-docstring
             return has_cmakelists
 
     kw["distclass"] = BinaryDistribution
 
-    print("")
+    print(flush=True)
 
-    return setuptools.setup(*args, **kw)
+    return setuptools.setup(**kw)
 
 
-def _collect_package_prefixes(package_dir, packages):
+def _collect_package_prefixes(
+    package_dir: Dict[str, str], packages: List[Union[Any, str]]
+) -> List[Union[Any, Tuple[str, str]]]:
     """
     Collect the list of prefixes for all packages
 
@@ -784,27 +781,25 @@ def _collect_package_prefixes(package_dir, packages):
     "top.not_a_subpackage" instead of "top", proper -- had such a package been
     specified.
     """
-    return list(
-        sorted(
-            ((package_dir[package].replace(".", "/"), package) for package in packages),
-            key=lambda tup: len(tup[0]),
-            reverse=True,
-        )
+    return sorted(
+        ((package_dir[package].replace(".", "/"), package) for package in packages),
+        key=lambda tup: len(tup[0]),
+        reverse=True,
     )
 
 
 def _classify_installed_files(
-    install_paths,
-    package_data,
-    package_prefixes,
-    py_modules,
-    new_py_modules,
-    scripts,
-    new_scripts,
-    data_files,
-    cmake_source_dir,
-    _cmake_install_dir,
-):
+    install_paths: Sequence[str],
+    package_data: Dict[str, List[str]],
+    package_prefixes: Sequence[Tuple[str, str]],
+    py_modules: Sequence[str],
+    new_py_modules: Dict[str, bool],
+    scripts: Sequence[str],
+    new_scripts: Dict[str, bool],
+    data_files: Dict[Any, Any],
+    cmake_source_dir: str,
+    _cmake_install_dir: str,
+) -> None:
     assert not os.path.isabs(cmake_source_dir)
     assert cmake_source_dir != "."
 
@@ -813,13 +808,12 @@ def _classify_installed_files(
         # if this installed file is not within the project root, complain and
         # exit
         if not to_platform_path(path).startswith(CMAKE_INSTALL_DIR()):
-            raise SKBuildError(
-                (
-                    "\n  CMake-installed files must be within the project root.\n"
-                    "    Project Root  : {}\n"
-                    "    Violating File: {}\n"
-                ).format(install_root, to_platform_path(path))
+            msg = (
+                "\n  CMake-installed files must be within the project root.\n"
+                f"    Project Root  : {install_root}\n"
+                f"    Violating File: {to_platform_path(path)}\n"
             )
+            raise SKBuildInvalidFileInstallationError(msg)
 
         # peel off the 'skbuild' prefix
         path = to_unix_path(os.path.relpath(path, CMAKE_INSTALL_DIR()))
@@ -829,7 +823,16 @@ def _classify_installed_files(
         )
 
 
-def _classify_file(path, package_data, package_prefixes, py_modules, new_py_modules, scripts, new_scripts, data_files):
+def _classify_file(
+    path: str,
+    package_data: Dict[str, List[str]],
+    package_prefixes: Sequence[Tuple[str, str]],
+    py_modules: Sequence[str],
+    new_py_modules: Dict[str, bool],
+    scripts: Sequence[str],
+    new_scripts: Dict[str, bool],
+    data_files: Dict[str, Set[str]],
+) -> None:
     found_package = False
     found_module = False
     found_script = False
@@ -842,7 +845,7 @@ def _classify_file(path, package_data, package_prefixes, py_modules, new_py_modu
             # peel off the package prefix
             path = to_unix_path(os.path.relpath(path, prefix))
 
-            package_file_list = package_data.get(package, [])
+            package_file_list = list(package_data.get(package, []))
             package_file_list.append(path)
             package_data[package] = package_file_list
 
@@ -889,7 +892,7 @@ def _classify_file(path, package_data, package_prefixes, py_modules, new_py_modu
     file_set.add(os.path.join(CMAKE_INSTALL_DIR(), path))
 
 
-def _copy_file(src_file, dest_file, hide_listing=True):
+def _copy_file(src_file: str, dest_file: str, hide_listing: Union[bool, int] = True) -> None:
     """Copy ``src_file`` to ``dest_file`` ensuring parent directory exists.
 
     By default, message like `creating directory /path/to/package` and
@@ -901,17 +904,24 @@ def _copy_file(src_file, dest_file, hide_listing=True):
     dest_dir = os.path.dirname(dest_file)
     if dest_dir != "" and not os.path.exists(dest_dir):
         if not hide_listing:
-            print("creating directory {}".format(dest_dir))
+            print(f"creating directory {dest_dir}", flush=True)
         mkdir_p(dest_dir)
 
     # Copy file
     if not hide_listing:
-        print("copying {} -> {}".format(src_file, dest_file))
-    copyfile(src_file, dest_file)
-    copymode(src_file, dest_file)
+        print(f"copying {src_file} -> {dest_file}", flush=True)
+    shutil.copyfile(src_file, dest_file)
+    shutil.copymode(src_file, dest_file)
 
 
-def _consolidate_package_modules(cmake_source_dir, packages, package_dir, py_modules, package_data, hide_listing):
+def _consolidate_package_modules(
+    cmake_source_dir: str,
+    packages: List[Union[Any, str]],
+    package_dir: Dict[str, str],
+    py_modules: List[Union[Any, str]],
+    package_data: Dict[str, List[str]],
+    hide_listing: Union[bool, int],
+) -> None:
     """This function consolidates packages having modules located in
     both the source tree and the CMake install tree into one location.
 
@@ -943,9 +953,9 @@ def _consolidate_package_modules(cmake_source_dir, packages, package_dir, py_mod
             packages, package_dir, py_modules, alternative_build_base=CMAKE_INSTALL_DIR()
         ).find_all_modules()
     except DistutilsError as msg:
-        raise SystemExit("error: {}".format(str(msg)))
+        raise SystemExit(f"error: {msg}") from None
 
-    print("")
+    print(flush=True)
 
     for entry in modules:
 
@@ -980,7 +990,11 @@ def _consolidate_package_modules(cmake_source_dir, packages, package_dir, py_mod
             package_data[package] = [stripped_module_file]
 
 
-def _consolidate_package_data_files(original_package_data, package_prefixes, hide_listing):
+def _consolidate_package_data_files(
+    original_package_data: Dict[str, List[str]],
+    package_prefixes: List[Union[Any, Tuple[str, str]]],
+    hide_listing: Union[bool, int],
+) -> None:
     """This function copies package data files specified using the ``package_data`` keyword
     into :func:`.constants.CMAKE_INSTALL_DIR()`.
 
@@ -1005,7 +1019,7 @@ def _consolidate_package_data_files(original_package_data, package_prefixes, hid
         raw_patterns = original_package_data[package]
         for pattern in raw_patterns:
             expanded_package_dir = os.path.join(project_root, prefix, pattern)
-            for src_data_file in glob(expanded_package_dir):
+            for src_data_file in glob.glob(expanded_package_dir):
                 full_prefix_length = len(os.path.join(project_root, prefix)) + 1
                 data_file = src_data_file[full_prefix_length:]
                 dest_data_file = os.path.join(CMAKE_INSTALL_DIR(), prefix, data_file)
