@@ -1,3 +1,5 @@
+from types import SimpleNamespace
+
 import pytest
 
 from skbuild.utils import distribution_hide_listing, distutils_log
@@ -5,10 +7,6 @@ from skbuild.utils import distribution_hide_listing, distutils_log
 setuptools_logging = pytest.importorskip("setuptools.logging")
 
 
-class SimpleNamespace(object):
-    pass
-
-
 def test_hide_listing(caplog):
     setuptools_logging.configure()
 
